@@ -67,6 +67,7 @@ public:
     }
     ~BackendPrivate()
     {
+        qDeleteAll(packages);
         delete cache;
         delete records;
         delete config;
@@ -209,6 +210,7 @@ void Backend::reloadCache()
 
     int packageCount = depCache->Head().PackageCount;
     d->packagesIndex.resize(packageCount);
+    d->packagesIndex.fill(-1);
     d->packages.reserve(packageCount);
 
     // Populate internal package cache
@@ -286,8 +288,8 @@ void Backend::reloadCache()
 
         pkgTagSection tags;
         while (tagFile.Step(tags)) {
-            QLatin1String name(tags.FindS("Package").c_str());
-            Package *pkg = package(name);
+            string name = tags.FindS("Package");
+            Package *pkg = package(QLatin1String(name.c_str()));
             if (pkg) {
                 pkg->setPinned(true);
             }
@@ -327,7 +329,7 @@ Package *Backend::package(pkgCache::PkgIterator &iter) const
     Q_D(const Backend);
 
     int index = d->packagesIndex.at(iter->ID);
-    if (index != -1) {
+    if (index != -1 && index < d->packages.size()) {
         return d->packages.at(index);
     }
     return 0;
