@@ -296,9 +296,8 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
                 m_cache->depCache()->MarkInstall(iter, true, 0, fromUser);
                 if (!State.Install() || m_cache->depCache()->BrokenCount() > 0) {
                     pkgProblemResolver Fix(m_cache->depCache());
-                    Fix.Clear(iter);
                     Fix.Protect(iter);
-                    Fix.Resolve(true);
+                    Fix.ResolveByKeep();
                 }
                 break;
             }
@@ -344,8 +343,29 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
         mapIter++;
     }
 
+    if (_error->PendingError() && (m_cache->depCache()->BrokenCount() == 0))
+    {
+        // We had dep errors, but fixed them
+        _error->Discard();
+        qDebug() << "fixed errors";
+    }
+
     emit workerStarted();
 
+    if (_error->PendingError()) {
+        string message;
+        QVariantMap details;
+
+        _error->PopMessage(message);
+        details[QLatin1String("FromWorker")] = true;
+        details[QLatin1String("ErrorText")] = QString::fromStdString(message);
+
+        emit errorOccurred(QApt::InitError, details);
+        emit workerFinished(false);
+        m_timeout->start();
+        return;
+    }
+
     // Lock the archive directory
     FileFd Lock;
     if (_config->FindB("Debug::NoLocking",false) == false)
@@ -469,6 +489,24 @@ void QAptWorker::commitChanges(QMap<QString, QVariant> instructionsList)
         return;
     }
 
+    bool failed = false;
+    for (pkgAcquire::ItemIterator i = fetcher.ItemsBegin(); i != fetcher.ItemsEnd(); ++i) {
+        if((*i)->Status == pkgAcquire::Item::StatDone && (*i)->Complete)
+            continue;
+        if((*i)->Status == pkgAcquire::Item::StatIdle)
+            continue;
+
+        failed = true;
+        break;
+    }
+
+    if (failed && !packageManager->FixMissing()) {
+        emit errorOccurred(QApt::FetchError, QVariantMap());
+        emit workerFinished(false);
+        m_timeout->start();
+        return;
+    }
+
     emit workerEvent(QApt::PackageDownloadFinished);
 
     emit workerEvent(QApt::CommitChangesStarted);
